@@ -1,7 +1,7 @@
 """
 monobit.bmfont - AngelCode BMFont format
 
-(c) 2019--2022 Rob Hagemans
+(c) 2019--2023 Rob Hagemans
 licence: https://opensource.org/licenses/MIT
 """
 
@@ -23,7 +23,6 @@ from ..streams import FileFormatError
 from ..binary import int_to_bytes, bytes_to_int
 from ..struct import little_endian as le
 from ..properties import reverse_dict
-from .. import struct
 from ..storage import loaders, savers
 from ..font import Font, Coord
 from ..glyph import Glyph
@@ -109,7 +108,7 @@ def _info(size):
         spacingHoriz='uint8',
         spacingVert='uint8',
         outline='uint8',
-        fontName = struct.char * (size-14),
+        fontName = le.char * (size-14),
     )
 
 # info bitfield
@@ -171,7 +170,7 @@ _COMMON = le.Struct(
 def _pages(npages, size):
     strlen = size // npages
     return le.Struct(
-        pageNames=(struct.char * strlen) * int(npages)
+        pageNames=(le.char * strlen) * int(npages)
     )
 
 
@@ -656,17 +655,17 @@ def _create_spritesheets(font, size=(256, 256), packed=False):
                 # > position to find the left position where the character should be drawn.
                 # > A negative value here would mean that the character slightly overlaps
                 # > the previous character.
-                xoffset=font.left_bearing + glyph.left_bearing + glyph.padding.left,
+                xoffset=cropped.left_bearing,
                 # > The `yoffset` gives the distance from the top of the cell height to the top
                 # > of the character. A negative value here would mean that the character extends
                 # > above the cell height.
-                yoffset=font.raster.top-(glyph.height+glyph.shift_up) + glyph.padding.top,
+                yoffset=font.raster.top-(cropped.height+cropped.shift_up),
                 # xadvance is the advance width from origin to next origin
                 # > The filled red dot marks the current cursor position, and the hollow red dot
                 # > marks the position of the cursor after drawing the character. You get to this
                 # > position by moving the cursor horizontally with the xadvance value.
                 # > If kerning pairs are used the cursor should also be moved accordingly.
-                xadvance=font.left_bearing + glyph.advance_width + font.right_bearing,
+                xadvance=cropped.advance_width,
                 page=page_id,
                 chnl=channels,
             ))
