@@ -1,7 +1,7 @@
 """
 monobit.formats.figlet - FIGlet .flf format
 
-(c) 2021--2022 Rob Hagemans
+(c) 2021--2023 Rob Hagemans
 licence: https://opensource.org/licenses/MIT
 """
 
@@ -244,17 +244,20 @@ def _convert_to_flf(font, hardblank='$'):
     # expand glyphs by bearings
     glyphs = [
         _g.expand(
-            left=max(0, font.left_bearing + _g.left_bearing),
-            bottom=max(0, font.shift_up + _g.shift_up),
-            right=max(0, font.right_bearing + _g.right_bearing),
+            left=max(0, _g.left_bearing),
+            bottom=max(0, _g.shift_up),
+            right=max(0, _g.right_bearing),
             # include leading; ensure glyphs are equal height
-            top=max(0, font.line_height - _g.height - max(0, font.shift_up + _g.shift_up)),
+            top=max(0, font.line_height - _g.height - max(0, _g.shift_up)),
         )
         for _g in glyphs
     ]
     return glyphs, props, font.get_comment()
 
-def _write_flf(outstream, flf_glyphs, flf_props, comments, ink='#', paper=' ', hardblank='$'):
+def _write_flf(
+        outstream, flf_glyphs, flf_props, comments,
+        ink='#', paper=' ', hardblank='$'
+    ):
     """Write out a figlet font file."""
     # header
     header = _FLF_HEADER(**vars(flf_props))
