@@ -1,7 +1,7 @@
 """
 monobit.formats.image - fonts stored in image files
 
-(c) 2019--2022 Rob Hagemans
+(c) 2019--2023 Rob Hagemans
 licence: https://opensource.org/licenses/MIT
 """
 
@@ -20,7 +20,7 @@ from ..storage import loaders, savers
 from ..streams import FileFormatError
 from ..font import Font
 from ..glyph import Glyph
-from ..renderer import chart, to_image, traverse_chart
+from ..chart import chart, traverse_chart
 
 
 DEFAULT_IMAGE_FORMAT = 'png'
@@ -193,9 +193,9 @@ if Image:
         """
         if len(fonts) > 1:
             raise FileFormatError('Can only save one font to image file.')
-        img = to_image(
-            chart(fonts[0], columns, margin, padding, scale, order, direction),
-            border, paper, ink
+        img = (
+            chart(fonts[0], columns, margin, padding, scale, order, direction)
+            .as_image(border=border, paper=paper, ink=ink)
         )
         try:
             img.save(outfile, format=image_format or Path(outfile).suffix[1:])
