@@ -1,6 +1,6 @@
 """
 Apply operation to bitmap font
-(c) 2019--2022 Rob Hagemans, licence: https://opensource.org/licenses/MIT
+(c) 2019--2023 Rob Hagemans, licence: https://opensource.org/licenses/MIT
 """
 
 import sys
@@ -9,7 +9,9 @@ from types import SimpleNamespace as Namespace
 from pathlib import Path
 
 import monobit
-from monobit.scripting import wrap_main, parse_subcommands, print_help, argrecord
+from monobit.scripting import (
+    wrap_main, parse_subcommands, print_help, argrecord, GLOBAL_ARG_PREFIX
+)
 
 script_name = Path(sys.argv[0]).name
 
@@ -29,7 +31,7 @@ global_options = {
 usage = (
     f'usage: {script_name} '
     + '[INFILE] [LOAD-OPTIONS] '
-    + ' '.join(f'[--{_op}]' for _op in global_options)
+    + ' '.join(f'[{GLOBAL_ARG_PREFIX}{_op}]' for _op in global_options)
     + ' [COMMAND [OPTION...]] ...'
     + ' [to [OUTFILE] [SAVE-OPTIONS]]'
 )
@@ -81,6 +83,12 @@ def main():
                 ):
                 command_args[0].command = 'load'
                 command_args[0].func = operations['load']
+                # special case `convert infile outfile` for convenience
+                if len(command_args[0].args) == 2:
+                    command_args.append(argrecord(
+                        command='save', func=operations['save'],
+                        args=[command_args[0].args.pop()])
+                    )
             # ensure last command is save
             if command_args[-1].command not in ('to', 'save'):
                 command_args.append(argrecord(command='save', func=operations['save']))
@@ -93,8 +101,8 @@ def main():
                 operation = operations[args.command]
                 if operation == monobit.load:
                     fonts += operation(*args.args, **args.kwargs)
-                elif operation == monobit.save:
-                    operation(fonts, *args.args, **args.kwargs)
+                elif operation.pack_operation:
+                    fonts = operation(fonts, *args.args, **args.kwargs)
                 else:
                     fonts = tuple(
                         operation(_font, *args.args, **args.kwargs)
